//
//  MTRectTools.m
//  CustomView
//
//  Created by zenomt on Tue Aug 07 2001.
//  Copyright (c) 2001 __CompanyName__. All rights reserved.
//

#import "MTRectTools.h"


@implementation MTRectTools

+ (NSRect) makeRectFromPoint:(NSPoint) firstPoint toPoint:(NSPoint) otherPoint
{
	NSRect rv;
	
	rv.origin.x = firstPoint.x;
	rv.origin.y = firstPoint.y;
	rv.size.width = otherPoint.x - firstPoint.x;
	rv.size.height = otherPoint.y - firstPoint.y;
	
	return rv;
}

+ (NSRect) normalizeRect:(NSRect) theRect
{
	NSRect rv = theRect;
	
	if (theRect.size.width < 0.0)
	{
		rv.origin.x += theRect.size.width;
		rv.size.width = 0.0 - theRect.size.width;
	}
	
	if (theRect.size.height < 0.0)
	{
		rv.origin.y += theRect.size.height;
		rv.size.height = 0.0 - theRect.size.height;
	}
	
	return rv;
}

+ (NSRect) _constrainRect:(NSRect)theRect toAspect:(NSSize)theAspect collapseAt:(double)collapse scaleByMax:(BOOL)shouldUseMax
{
	NSRect rv = theRect;
	double scaleFactor;
	double wScaleFactor, hScaleFactor;
	double normalizedWidth, normalizedHeight;
	
	normalizedWidth = fabs(theAspect.width);
	normalizedHeight = fabs(theAspect.height);
	
	if (normalizedWidth == 0.0 || normalizedHeight == 0.0)
		return theRect; // punt
	
	if (shouldUseMax)
		scaleFactor = MAX (
			(fabs(theRect.size.width) / normalizedWidth),
			(fabs(theRect.size.height) / normalizedHeight));
	else
		scaleFactor = MIN (
			(fabs(theRect.size.width) / normalizedWidth),
			(fabs(theRect.size.height) / normalizedHeight));
	if (((scaleFactor * normalizedWidth) < collapse) || ((scaleFactor * normalizedHeight ) < collapse))
		scaleFactor = 0.0;

	wScaleFactor = hScaleFactor = scaleFactor;
	if (theRect.size.width < 0.0)
		wScaleFactor = 0.0 - wScaleFactor;
	if (theRect.size.height < 0.0)
		hScaleFactor = 0.0 - hScaleFactor;
	rv.size.width = normalizedWidth * wScaleFactor;
	rv.size.height = normalizedHeight * hScaleFactor;
	return rv;
}

+ (NSRect) constrainRect:(NSRect)theRect toAspect:(NSSize)theAspect collapseAt:(double)collapse
{
	return [self _constrainRect:theRect toAspect:theAspect collapseAt:collapse scaleByMax:(BOOL)YES];
}

// origin of theRect must already be in the frame.  frameRect must be normalized
+ (NSRect) scaleRect:(NSRect)theRect toFit:(NSRect)frameRect
{
	NSRect newRect;
	double farX, farY;
	double farFrameX, farFrameY;
	
	farX = theRect.origin.x + theRect.size.width;
	farY = theRect.origin.y + theRect.size.height;
	farFrameX = frameRect.origin.x + frameRect.size.width;
	farFrameY = frameRect.origin.y + frameRect.size.height;
	
	if (farX < frameRect.origin.x)
		farX = frameRect.origin.x;
	else if (farX > farFrameX)
		farX = farFrameX;
	if (farY < frameRect.origin.y)
		farY = frameRect.origin.y;
	else if (farY > farFrameY)
		farY = farFrameY;
	
	newRect.origin.x = theRect.origin.x;
	newRect.origin.y = theRect.origin.y;
	newRect.size.width = farX - theRect.origin.x;
	newRect.size.height = farY - theRect.origin.y;
	return [self _constrainRect:newRect toAspect:theRect.size collapseAt:0.0 scaleByMax:NO];
}

@end
