#import "MyView.h"
#include <unistd.h>
#include "mandel.h"
#import "MTRectTools.h"

#define kCollapseDrag 12.0

@implementation MyView

- (void)doReportButton:(id)sender
{
	int x;
	int firstDrawn = 0;
	
	for (x=0; x < height; x++)
	{
		if(drawnTable[x] == 0)
		{
			if (firstDrawn)
				printf("line %d not drawn!\n", x);
		}
		else
		{
			firstDrawn = 1;
		}
	}
}

- (void)drawRect:(NSRect)rect
{
    // NSGraphicsContext *currentContext = [NSGraphicsContext currentContext];
    NSRect currentFrame;
    int needsStartButton = 0;
    // NSAffineTransform * myTransform;
    int x;
    
    currentFrame = [self bounds];
    [workLock lock];
    
    if ((myImage != nil) && ! (
	( currentFrame.size.width == frame.size.width ) &&
	( currentFrame.size.height == frame.size.height )
    )) // resized!
    {
	comp_sequence++; // invalidate any current work
	[myImage release];
	myImage = nil;
	viewX -= viewStep * ( currentFrame.size.width - frame.size.width ) / 2.0 ;
	viewY += viewStep * ( currentFrame.size.height - frame.size.height ) / 2.0 ;
	height = currentFrame.size.height ;
	needsStartButton = 1;
    }

    if (myImage == nil)
    {	
    
        frame = currentFrame;
        window = [self window];

        myImage = [NSBitmapImageRep alloc];
        [myImage
            initWithBitmapDataPlanes:NULL
            pixelsWide:frame.size.width pixelsHigh:frame.size.height
            bitsPerSample:8 samplesPerPixel:3 hasAlpha:NO isPlanar:NO
            colorSpaceName:NSCalibratedRGBColorSpace
            bytesPerRow:(frame.size.width*3) bitsPerPixel:0
        ];
        bitmapData = [myImage bitmapData];

        for (x=0; x < (frame.size.width * frame.size.height * 3); x++)
        {
            bitmapData[x] = 255;
        }
    }
    
    // printf("rect: y = %f   height = %f\n", rect.origin.y, rect.size.height );
    if (rect.size.height < height) // only pay attention to partial drawing
    {
	for(x=0; x < (int)rect.size.height; x++) drawnTable[(int)rect.origin.y + x] = 1;
    }

    // [currentContext saveGraphicsState];
    // myTransform = [NSAffineTransform transform];
    // [myTransform rotateByDegrees:5];
    // [myTransform concat];
    
    [myImage drawInRect:currentFrame];
    if (isDragging)
    {
	[[[NSColor selectedTextBackgroundColor] colorWithAlphaComponent:0.35] set];
	NSRectFillUsingOperation (mouseDragRect, NSCompositeSourceOver);
	[[NSColor selectedTextBackgroundColor] set];
	NSFrameRect (mouseDragRect);
    }
    [workLock unlock];
    
    // [currentContext restoreGraphicsState];
    if (needsStartButton)
	[self startButton:nil];
}

- (BOOL) acceptsFirstMouse:(NSEvent *)theEvent { return NO; }
- (BOOL) acceptsFirstResponder { return YES; }
- (BOOL) isOpaque { return YES; }
- (BOOL) isFlipped { return NO; }


- (void)mouseDown:(NSEvent *)theEvent
{
	mouseDownPoint = [self convertPoint:[theEvent locationInWindow] fromView:nil];
	isSelecting = YES; isDragging = NO;
}

- (void)mouseUp:(NSEvent *)theEvent
{
	double newX, newY;

	if (isDragging && mouseDragRect.size.width < kCollapseDrag )
	{
		isSelecting = NO; isDragging = NO;
		return;
	}

	if (isDragging)
	{
		newX = viewX + ( viewStep * mouseDragRect.origin.x );
		newY = viewY - ( viewStep * (frame.size.height - (mouseDragRect.origin.y + mouseDragRect.size.height)));
	}
	else
	{
		newX = viewX + ( viewStep * mouseDownPoint.x );
		newY = viewY - ( viewStep * (frame.size.height - mouseDownPoint.y ));
	}
	
	[workLock lock];
	if ( [originSwitch state] == NSOnState )
	{
		originX = newX;
		originY = newY;
	}
	else
	{
		if (isDragging)
		{
			viewX = newX;
			viewY = newY;
			
			viewStep = viewStep * (mouseDragRect.size.width / frame.size.width);
		}
		else
		{
			viewX = newX - ( viewStep * frame.size.width * 0.25 );
			viewY = newY + ( viewStep * frame.size.height * 0.25 );
	
			viewStep = viewStep * 0.5 ;
		}
	}
	[workLock unlock];

	isSelecting = NO; isDragging = NO;
	[self startButton:nil];
}

- (void)mouseDragged:(NSEvent *)theEvent
{
	if (isDragging)
		[self setNeedsDisplayInRect:mouseDragRect];
	isDragging = YES;
	mouseDragPoint = [self convertPoint:[theEvent locationInWindow] fromView:nil];
	mouseDragRect = [MTRectTools makeRectFromPoint:mouseDownPoint toPoint:mouseDragPoint];
	mouseDragRect = [MTRectTools constrainRect:mouseDragRect toAspect:frame.size collapseAt:kCollapseDrag];
	mouseDragRect = [MTRectTools scaleRect:mouseDragRect toFit:frame];
	mouseDragRect = [MTRectTools normalizeRect:mouseDragRect];
	[self setNeedsDisplayInRect:mouseDragRect];
}

- (void)keyDown:(NSEvent *)theEvent
{
    NSBezierPath * myPath;
    NSGraphicsContext *currentContext ;
        
    printf ("keydown\n");
    [self lockFocus];
    currentContext = [NSGraphicsContext currentContext];

    myPath = [NSBezierPath bezierPath];
    [myPath setLineWidth:20];
    [myPath moveToPoint:NSMakePoint(-100.0,-100.0)];
    [myPath lineToPoint:NSMakePoint(200.0,300.0)];
    [myPath stroke];
    [currentContext flushGraphics];
    [self unlockFocus];
    // printf ("keyDown\n");
}

- (void)keyUp:(NSEvent *)theEvent
{
    // printf ("keyUp\n");
}

- (void) doThreadMenu:(id)sender
{
    num_threads = atoi([[sender titleOfSelectedItem] cString]);
    // printf ("threads: %d\n", num_threads );
}

- (void) doDwellMenu:(id)sender
{
    max_dwells = atoi([[sender titleOfSelectedItem] cString]);
    // printf ("max dwells: %d\n", max_dwells);
}

- (void) doResetOriginButton:(id)sender
{
    originX = 0.0;
    originY = 0.0;
    [self startButton:sender];
}

- (void) resetButton:(id)sender
{
    [workLock lock];
    viewX = -2.5; viewStep = 4.0 / frame.size.width;
    viewY = (double) frame.size.height * viewStep * 0.5;
    [workLock unlock];
    [self startButton:sender];
}

- (void) startButton:(id)sender
{
    int x, randIndex, tmp;
    NSNumber * currentSequence;
    
    
    // printf ("\n\n\n");
    
    [workLock lock];
    comp_sequence++;
    lastLine = 0;
    total_dwells = 0;
    acc_dwells = 0;
    escapeRadius = [escapeRadiusSlider floatValue];
    currentSequence = [[NSNumber alloc] initWithInt:comp_sequence];
    [workLock unlock];
 
    if ([clearSwitch state] == NSOnState)
    {
	for (x=0; x < (frame.size.width * frame.size.height * 3); x++)
	{
	    bitmapData[x] = 255;
	}
    }
    
    [self displayRect:frame];
    
    for (x = 0; x < height; x++) drawnTable[x] = 0;
    for (x = 0; x < height; x++) lineTable[x] = x;
    if ([randomizerSwitch state] == NSOnState) {
	for (x = 0; x < height; x++)
	{
	    randIndex = rand() % height;
	    tmp = lineTable[x];
	    lineTable[x] = lineTable[randIndex];
	    lineTable[randIndex] = tmp;
	}
    }


    for (x = 0; x < num_threads; x++)
        [NSThread detachNewThreadSelector:@selector(threadMain:) toTarget:self withObject:[currentSequence retain]];
    [currentSequence release];
}

- (void)threadMain:(NSNumber *)currentSequence
{
    int w, h;
    int buf[16384]; // BUG!
    int update_dwells;
    double x, y, step, currentY;
    int idx, bitmapIdx;
    int working_sequence;
    int myLine;
    NSRect updateRect;
    
    x = viewX; y = viewY;
    w = frame.size.width;
    h = frame.size.height;
    
    updateRect.origin.x = 0;
    updateRect.size.width = w;
    updateRect.size.height = 1;
    
    step = viewStep;
    // printf ("viewX: %f  viewY: %f  viewStep: %f\n", x, y, step );

    working_sequence = [currentSequence intValue];
    [currentSequence release];
    
    while (1)
    {
	[workLock lock];
	if (working_sequence != comp_sequence)
	{
		[workLock unlock];
		return;
	}
	myLine = lastLine++;
	[workLock unlock];
	if (myLine >= h)
	    break;

	currentY = y - (step * lineTable[myLine]);
	
        update_dwells = mandelLine ( originX, originY, escapeRadius, x, currentY, step, max_dwells, w, buf );
	[workLock lock];
	if (working_sequence != comp_sequence)
	{
	    [workLock unlock];
	    return;
	}
	total_dwells += update_dwells; acc_dwells += update_dwells;
		
        for (idx = 0, bitmapIdx = lineTable[myLine] * w * 3; idx < w; idx++, bitmapIdx += 3 )
        {
            if (buf[idx] < 0)
            {
                bitmapData[bitmapIdx    ] = 0;
                bitmapData[bitmapIdx + 1] = 0;
                bitmapData[bitmapIdx + 2] = 0;
            }
            else
            {
                bitmapData[bitmapIdx    ] = ( buf[idx] % 16) * 16;
                bitmapData[bitmapIdx + 1] = (( buf[idx] / 16 ) % 16 ) * 16;
                bitmapData[bitmapIdx + 2] = (( buf[idx] / 256 ) % 16 ) * 16;
            }
        }
	/*
        if ( acc_dwells > 10000000 )
        {
            [self setNeedsDisplay:YES];
            acc_dwells = 0;
        }
	*/
	updateRect.origin.y = h - lineTable[myLine];
	[self setNeedsDisplayInRect:updateRect];
	[workLock unlock];
    }
    
    // [workLock lock];
    [self setNeedsDisplay:YES];
    // [workLock unlock];
    sleep(1);
    [self setNeedsDisplay:YES];
    // printf ("exiting thread\n");
    return;
}

// delegate of NSApplication
- (void)applicationDidFinishLaunching:(NSNotification *)aNotification
{
    // for some reason, app's window doesn't start out being key and front.  :(
    // printf ("applicationDidFinishLaunching\n");
    [window makeKeyAndOrderFront:nil];
    [window setInitialFirstResponder:self];
    num_threads = atoi([[threadMenu titleOfSelectedItem] cString]);;
    max_dwells = atoi([[dwellMenu titleOfSelectedItem] cString]);;
    lastLine = 0;
    height = frame.size.height;
    comp_sequence = 0;
    // viewX = -2.5; viewStep = 4.0 / frame.size.width;
    // viewY = (double) frame.size.height * viewStep * 0.5;
    originX = 0.0;
    originY = 0.0;
    workLock = [[NSLock alloc] init];
    isSelecting = NO; isDragging = NO;
    [self resetButton:nil];
    //    [NSThread detachNewThreadSelector:@selector(threadMain) toTarget:self withObject:nil];

}

- (BOOL)applicationShouldTerminateAfterLastWindowClosed:(NSApplication *)theApplication
{
    return YES;
}

@end
